﻿using Dungeonator;
using MonoMod.RuntimeDetour;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Reflection;
using System.Text;
using UnityEngine;

namespace DadsGungeonPlus
{
    public static class Extensions
    {
        public static List<EnemyData> RandomiseListOrder(this List<EnemyData> oldList)
        {
            List<EnemyData> newList = new List<EnemyData>();
            int oldListcount = oldList.Count;
            for (int i = 0; i < oldListcount; i++)
            {
                if (oldList.Count > 0)
                {
                    EnemyData selectedData = BraveUtility.RandomElement(oldList);
                    if (selectedData != null)
                    {
                        newList.Add(selectedData);
                        oldList.Remove(selectedData);
                    }
                    else Debug.LogError("Randomly selected data in list randomiser was NULL!");
                }
                else Debug.LogError("Tried to randomly select from a list with no entries.");
            }
            return newList;
        }
    }
    class Handler
    {
        public static void Init()
        {
            EnemyDeathHook = new Hook(
                typeof(HealthHaver).GetMethod("Die", BindingFlags.Instance | BindingFlags.Public),
                typeof(Handler).GetMethod("onHealthHaverDie")
            );
            PlayerSpawnHook = new Hook(
                typeof(PlayerController).GetMethod("orig_Start", BindingFlags.Public | BindingFlags.Instance),
                typeof(Handler).GetMethod("DoSetup"));
        }
        public static Hook EnemyDeathHook;
        public static Hook PlayerSpawnHook;
        public static void onHealthHaverDie(Action<HealthHaver, Vector2> orig, HealthHaver self, Vector2 finalDamageDir)
        {
            if (self.aiActor != null && self.specRigidbody != null && !string.IsNullOrEmpty(self.aiActor.EnemyGuid) && !bannedIDs.Contains(self.aiActor.EnemyGuid))
            {
                if (!self.IsBoss || !IgnoreBosses)
                {
                    if (self.GetComponent<IsDadsHomeSpawn>() == null && self.GetComponent<DisplacedImageController>() == null && self.GetComponent<MirrorImageController>() == null)
                    {
                        EnemyData newStoredEnemy = new EnemyData();
                        newStoredEnemy.guid = self.aiActor.EnemyGuid;
                        newStoredEnemy.charmed = self.aiActor.CanTargetEnemies;
                        newStoredEnemy.roomClear = !self.aiActor.IgnoreForRoomClear;
                        newStoredEnemy.maxHP = self.GetMaxHealth();
                        newStoredEnemy.isJammed = self.aiActor.IsBlackPhantom;
                        storedEnemies.Add(newStoredEnemy);
                    }
                }
            }
            orig(self, finalDamageDir);
        }
        public static void DoSetup(Action<PlayerController> action, PlayerController player)
        {
            action(player);
            if (player.GetComponent<PlayerDadsHomeComponent>() == null) player.gameObject.GetOrAddComponent<PlayerDadsHomeComponent>();
        }
        public static void SpawnBonusEnemies(RoomHandler room)
        {
            try
            {

                if (room.HasActiveEnemies(RoomHandler.ActiveEnemyType.RoomClear) || SpawnOnEmptyRooms)
                {
                    if (!room.hasEverBeenVisited || SpawnOnPreviouslyVisitedRooms)
                    {
                        if (storedEnemies != null && storedEnemies.Count > 0)
                        {
                            float floatAmtToSpawn = storedEnemies.Count * ((float)SpawnPercentage / 100);
                            int amtToSpawn = Mathf.CeilToInt(floatAmtToSpawn);
                            if (SpawnPercentage >= 100) amtToSpawn = storedEnemies.Count;

                            List<EnemyData> shuffledStoredEnemies = new List<EnemyData>();
                            shuffledStoredEnemies.AddRange(storedEnemies);
                            shuffledStoredEnemies = shuffledStoredEnemies.RandomiseListOrder();

                            for (int i = 0; i < amtToSpawn; i++)
                            {
                                if (shuffledStoredEnemies[i] != null)
                                {
                                    EnemyData data = shuffledStoredEnemies[i];
                                    var respawnedEnemy = EnemyDatabase.GetOrLoadByGuid(data.guid);
                                    if (respawnedEnemy != null)
                                    {
                                        IntVector2? bestRewardLocation = room.GetRandomVisibleClearSpot(2, 2);
                                        AIActor TargetActor = AIActor.Spawn(respawnedEnemy.aiActor, bestRewardLocation.Value, GameManager.Instance.Dungeon.data.GetAbsoluteRoomFromPosition(bestRewardLocation.Value), true, AIActor.AwakenAnimationType.Default, true);

                                        TargetActor.CanTargetEnemies = data.charmed;
                                        TargetActor.CanTargetPlayers = !data.charmed;
                                        if (data.isJammed) TargetActor.BecomeBlackPhantom();
                                        if (TargetActor.healthHaver)
                                        {
                                            TargetActor.healthHaver.SetHealthMaximum(data.maxHP, null, true);
                                            if (enemyHPMult != 100) TargetActor.healthHaver.SetHealthMaximum((TargetActor.healthHaver.GetMaxHealth() * ((float)enemyHPMult) / 100), null, true);
                                            if (TargetActor.healthHaver.IsBoss) TargetActor.healthHaver.SetHealthMaximum((TargetActor.healthHaver.GetMaxHealth() * ((float)bossHPMult) / 100), null, true);
                                        }
                                        PhysicsEngine.Instance.RegisterOverlappingGhostCollisionExceptions(TargetActor.specRigidbody, null, false);


                                        TargetActor.ToggleRenderers(false);
                                        TargetActor.invisibleUntilAwaken = true;
                                        if (TargetActor.EnemyGuid != "556e9f2a10f9411cb9dbfd61e0e0f1e1") TargetActor.specRigidbody.CollideWithOthers = false;
                                        TargetActor.IsGone = true;
                                        TargetActor.HandleReinforcementFallIntoRoom(UnityEngine.Random.Range(0f, 6f));

                                        if (TargetActor.GetComponent<SpawnEnemyOnDeath>() != null) UnityEngine.Object.Destroy(TargetActor.GetComponent<SpawnEnemyOnDeath>());
                                        TargetActor.gameObject.GetOrAddComponent<IsDadsHomeSpawn>();
                                    }
                                    else Debug.LogError("Could not find an enemy with the guid: " + data.guid);
                                }
                                else
                                {
                                    Debug.LogError("The Shuffled Stored Enemy at Index (" + i + ") was null!");
                                }
                                if (!room.IsSealed)
                                {
                                    if (room.area.runtimePrototypeData != null)
                                    {
                                        bool flag2 = false;
                                        for (int i2 = 0; i2 < room.area.runtimePrototypeData.roomEvents.Count; i2++)
                                        {
                                            RoomEventDefinition roomEventDefinition = room.area.runtimePrototypeData.roomEvents[i2];
                                            if (roomEventDefinition.condition == RoomEventTriggerCondition.ON_ENEMIES_CLEARED && roomEventDefinition.action == RoomEventTriggerAction.UNSEAL_ROOM)
                                            {
                                                flag2 = true;
                                            }
                                        }
                                        if (!flag2)
                                        {
                                            room.area.runtimePrototypeData.roomEvents.Add(new RoomEventDefinition(RoomEventTriggerCondition.ON_ENEMIES_CLEARED, RoomEventTriggerAction.UNSEAL_ROOM));
                                        }
                                    }
                                }
                                room.SealRoom();
                            }
                        }
                        //Spawn the enemies

                    }
                }
            }
            catch (Exception e)
            {
                ETGModConsole.Log(e.Message);
                ETGModConsole.Log(e.StackTrace);
            }
        }

        public static void ClearLists()
        {
            storedEnemies.Clear();
        }
        public static bool SpawnOnPreviouslyVisitedRooms;
        public static bool IgnoreBosses;
        public static bool SpawnOnEmptyRooms;
        public static int SpawnPercentage;
        public static int enemyHPMult;
        public static int bossHPMult;

        public static List<EnemyData> storedEnemies = new List<EnemyData>();

        public static List<string> bannedIDs = new List<string>()
        {
            "465da2bb086a4a88a803f79fe3a27677", //Dragun
            "05b8afe0b6cc4fffa9dc6036fa24c8ec", //Adv Dragun
            "41ee1c8538e8474a82a74c4aff99c712", //Heligunim
            "f3b04a067a65492f8b279130323b41f0", //Wallmonger
            "3f11bbbc439c4086a180eb0fb9990cb4", //Kill Pillars
            "6868795625bd46f3ae3e4377adce288b", //Res Rat
            "4d164ba3f62648809a4a82c90fc22cae", //Rat Mech
            "22fc2c2c45fb47cf9fb5f7b043a70122", //Gripmaster
        };
    }
    public class EnemyData
    {
        public string guid;
        public bool charmed;
        public bool roomClear;
        public float maxHP;
        public bool isJammed;
    }
    public class IsDadsHomeSpawn : MonoBehaviour { };
    public class PlayerDadsHomeComponent : MonoBehaviour
    {
        public PlayerDadsHomeComponent()
        {

        }
        private RoomHandler lastRoom;
        private PlayerController self;
        private void Start()
        {
            self = base.GetComponent<PlayerController>();
            self.OnNewFloorLoaded += LoadedLevel;
        }
        private void LoadedLevel(PlayerController me)
        {
            if (GameManager.Instance.SecondaryPlayer != null && GameManager.Instance.SecondaryPlayer == self) return;
            if (GameManager.Instance.Dungeon.tileIndices.tilesetId == GlobalDungeonData.ValidTilesets.CASTLEGEON || GameManager.Instance.IsLoadingFirstShortcutFloor)
            {
                self.OnNewFloorLoaded -= LoadedLevel;
                Handler.ClearLists();
            }
        }
        public void Update()
        {
            if (self && self.CurrentRoom != null && !string.IsNullOrEmpty(self.CurrentRoom.GetRoomName()) && self.CurrentRoom != lastRoom)
            {
                if (!GameManager.Instance.IsFoyer)
                {
                    if (GameManager.Instance.SecondaryPlayer != null && GameManager.Instance.SecondaryPlayer == self) return;
                    Handler.SpawnBonusEnemies(self.CurrentRoom);
                    lastRoom = self.CurrentRoom;
                }
            }
        }
    };
}
