﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using UnityEngine;

namespace DadsGungeonPlus
{
    public class Module : ETGModule
    {
        public override void Start()
        {
            Handler.SpawnPercentage = 100;
            Handler.enemyHPMult = 100;
            Handler.bossHPMult = 25;
            Handler.Init();

            //COMMANDS

            ETGModConsole.Commands.AddGroup("dadsgungeon", delegate (string[] args)
            {
                if (dadsGungeonEnabled)
                {
                    ETGModConsole.Log("Dad's Gungeon+ is currently enabled with the following settings:");
                    ETGModConsole.Log(string.Format("Affects Bosses: <color=#ff0000ff>{0}</color>", !Handler.IgnoreBosses));
                    ETGModConsole.Log(string.Format("Spawn on Empty Rooms: <color=#ff0000ff>{0}</color>", Handler.SpawnOnEmptyRooms));
                    ETGModConsole.Log(string.Format("Spawn on Backtracking: <color=#ff0000ff>{0}</color>", Handler.SpawnOnPreviouslyVisitedRooms));
                    ETGModConsole.Log(string.Format("Percentage to Spawn: <color=#ff0000ff>{0}%</color>", Handler.SpawnPercentage));
                    ETGModConsole.Log(string.Format("Enemy HP Mult: <color=#ff0000ff>{0}%</color>", Handler.enemyHPMult));
                    ETGModConsole.Log(string.Format("Boss HP Mult: <color=#ff0000ff>{0}%</color>", Handler.bossHPMult));
                    ETGModConsole.Log("Type <color=#ff0000ff>dadsgungeon</color> in the console and press tab to see the commands you can use to change these values. \n");

                }
                else
                {
                    ETGModConsole.Log("Dad's Gungeon+ is currently disabled. Use the command <color=#ff0000ff>dadsgungeon toggle</color> to enable it!");

                }
            });
            ETGModConsole.Commands.GetGroup("dadsgungeon").AddUnit("toggle", delegate (string[] args)
            {
                if (dadsGungeonEnabled)
                {
                    ETGModConsole.Log("Dad's Gungeon+ has been disabled.");
                    dadsGungeonEnabled = false;
                }
                else
                {
                    ETGModConsole.Log("Dad's Gungeon+ has been enabled with the following settings:");
                    ETGModConsole.Log(string.Format("Affects Bosses: <color=#ff0000ff>{0}</color>", !Handler.IgnoreBosses));
                    ETGModConsole.Log(string.Format("Spawn on Empty Rooms: <color=#ff0000ff>{0}</color>", Handler.SpawnOnEmptyRooms));
                    ETGModConsole.Log(string.Format("Spawn on Backtracking: <color=#ff0000ff>{0}</color>", Handler.SpawnOnPreviouslyVisitedRooms));
                    ETGModConsole.Log(string.Format("Percentage to Spawn: <color=#ff0000ff>{0}%</color>", Handler.SpawnPercentage));
                    ETGModConsole.Log(string.Format("Enemy HP Mult: <color=#ff0000ff>{0}%</color>", Handler.enemyHPMult));
                    ETGModConsole.Log(string.Format("Boss HP Mult: <color=#ff0000ff>{0}%</color>", Handler.bossHPMult));
                    ETGModConsole.Log("Type <color=#ff0000ff>dadsgungeon</color> in the console and press tab to see the commands you can use to change these values.\n");
                    dadsGungeonEnabled = true;
                }
            });

            ETGModConsole.Commands.GetGroup("dadsgungeon").AddUnit("affects_bosses", delegate (string[] args)
            {
                if (Handler.IgnoreBosses)
                {
                    ETGModConsole.Log("Dad's Gungeon+ will now duplicate boss enemies.");
                    Handler.IgnoreBosses = false;
                }
                else
                {
                    ETGModConsole.Log("Dad's Gungeon+ will now no longer affect bosses.");
                    Handler.IgnoreBosses = true;
                }
            });

            ETGModConsole.Commands.GetGroup("dadsgungeon").AddUnit("spawn_on_empty_rooms", delegate (string[] args)
            {
                if (Handler.SpawnOnEmptyRooms)
                {
                    ETGModConsole.Log("Dad's Gungeon+ will no longer spawn enemies on empty rooms.");
                    Handler.SpawnOnEmptyRooms = false;
                }
                else
                {
                    ETGModConsole.Log("Dad's Gungeon+ will now spawn enemy waves in otherwise empty rooms.");
                    Handler.SpawnOnEmptyRooms = true;
                }
            });

            ETGModConsole.Commands.GetGroup("dadsgungeon").AddUnit("spawn_on_backtrack", delegate (string[] args)
            {
                if (Handler.SpawnOnPreviouslyVisitedRooms)
                {
                    ETGModConsole.Log("Dad's Gungeon+ will no longer spawn enemies on previously visited rooms.");
                    Handler.SpawnOnPreviouslyVisitedRooms = false;
                }
                else
                {
                    ETGModConsole.Log("Dad's Gungeon+ will now spawn enemy waves in rooms you've already been.");
                    Handler.SpawnOnPreviouslyVisitedRooms = true;
                }
            });

            ETGModConsole.Commands.GetGroup("dadsgungeon").AddUnit("spawn_percentage", delegate (string[] args)
            {
                if (args != null && args[0] != null)
                {
                    if (args.Count() > 1)
                    {
                        ETGModConsole.Log("ERROR: TOO MANY ARGUMENTS");
                    }
                    else
                    {
                        try
                        {
                            int spawnPercent = int.Parse(args[0]);
                            if (spawnPercent > 0 && spawnPercent < 101)
                            {
                                Handler.SpawnPercentage = spawnPercent;
                                ETGModConsole.Log(string.Format("Dad's Gungeon+ will spawn {0}% of saved enemies every room.", spawnPercent));

                            }
                            else
                            {
                                ETGModConsole.Log("ERROR: Spawn Percentage must be above 0, and below 100.");

                            }
                        }
                        catch (Exception e)
                        {
                            ETGModConsole.Log(e.ToString());
                        }
                    }
                }
            });

            ETGModConsole.Commands.GetGroup("dadsgungeon").AddUnit("enemy_hp_mult", delegate (string[] args)
            {
                if (args != null && args[0] != null)
                {
                    if (args.Count() > 1)
                    {
                        ETGModConsole.Log("ERROR: TOO MANY ARGUMENTS");
                    }
                    else
                    {
                        try
                        {
                            int spawnPercent = int.Parse(args[0]);
                            if (spawnPercent > 0 && spawnPercent < 101)
                            {
                                Handler.enemyHPMult = spawnPercent;
                                ETGModConsole.Log(string.Format("Enemies spawned by Dad's Gungeon+ will have {0}% their usual health.", spawnPercent));

                            }
                            else
                            {
                                ETGModConsole.Log("ERROR: Health multiplier must be above 0, and below 100.");

                            }
                        }
                        catch (Exception e)
                        {
                            ETGModConsole.Log(e.ToString());
                        }
                    }
                }
            });

            ETGModConsole.Commands.GetGroup("dadsgungeon").AddUnit("boss_hp_mult", delegate (string[] args)
            {
                if (args != null && args[0] != null)
                {
                    if (args.Count() > 1)
                    {
                        ETGModConsole.Log("ERROR: TOO MANY ARGUMENTS");
                    }
                    else
                    {
                        try
                        {
                            int spawnPercent = int.Parse(args[0]);
                            if (spawnPercent > 0 && spawnPercent < 101)
                            {
                                Handler.bossHPMult = spawnPercent;
                                ETGModConsole.Log(string.Format("Bosses spawned by Dad's Gungeon+ will have {0}% their usual health (on top of the enemy HP mult).", spawnPercent));

                            }
                            else
                            {
                                ETGModConsole.Log("ERROR: Boss Health mult must be above 0, and below 100.");

                            }
                        }
                        catch (Exception e)
                        {
                            ETGModConsole.Log(e.ToString());
                        }
                    }
                }
            });

            ETGModConsole.Log("Ready to go, daddy-o! (Dad's Gungeon+ Fully Initialised)");
        }
        public static bool dadsGungeonEnabled = false;
        public override void Exit() { }
        public override void Init() { }
    }
}
